import {Component} from '@angular/core'
import {CompanyService} from '../../service/company.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-add',
  templateUrl: './add.component.html',
  styleUrls: ['./add.component.sass']
})
export class AddComponent extends ErrorHandlingComponent {
  formFields: FormField[] = [
    {
      name: 'name',
      label: 'Nom',
      icon: 'form-textbox',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un nom est requis'}
      ]
    }
  ]
  submittedValues: any | null

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'exists-company-name',
    messageProducer: error => `Une bannière avec le nom '${error.name}' existe déjà`
  }]

  constructor(
    private companyService: CompanyService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouvelle bannière'
  }

  ngOnInit() {
    super.ngOnInit()
  }

  submit(values) {
    this.submittedValues = values
    this.subscribeAndNavigate(
      this.companyService.save(values.name),
      '/catalog/company/list'
    )
  }
}
