import {Component, ViewChild} from '@angular/core'
import {Validators} from '@angular/forms'
import {GroupService} from '../../services/group.service'
import {ActivatedRoute, Router} from '@angular/router'
import {currentPermissionsFieldComponent} from '../../../shared/components/permissions-field/permissions-field.component'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-add',
  templateUrl: './add.component.html',
  styleUrls: ['./add.component.sass']
})
export class AddComponent extends ErrorHandlingComponent {
  @ViewChild('permissionsTemplateRef', {static: true}) permissionsTemplateRef

  formFields: FormField[] = [{
    name: 'name',
    label: 'Nom',
    icon: 'account-multiple',
    type: 'text',
    required: true,
    validator: Validators.minLength(3),
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un nom est requis'},
      {conditionFn: errors => errors.minlength, message: 'Le nom d\'un groupe doit comprendre au moins 3 caractères'}
    ]
  }, {
    name: 'permissions',
    label: 'Permissions',
    type: 'permissions'
  }]

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'exists-group-name',
    messageProducer: error => `Un groupe avec le nom '${error.name}' existe déjà`
  }]

  constructor(
    private groupService: GroupService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouveau groupe'
  }

  ngOnInit() {
    super.ngOnInit();

    this.formFields[this.formFields.length - 1].template = this.permissionsTemplateRef
  }

  submit(values) {
    const permissionsField = currentPermissionsFieldComponent
    if (permissionsField.valid()) {
      this.subscribeAndNavigate(
        this.groupService.save(values.name, permissionsField.allEnabledPermissions),
        '/admin/group/list'
      )
    }
  }
}
