import {Component, Input, OnDestroy, OnInit} from '@angular/core';
import {User, Permission} from "../../model/user";
import {AccountService} from "../../../accounts/services/account.service";
import {ActivatedRoute, Router} from "@angular/router";
import {takeUntil} from "rxjs/operators";
import {AppState} from "../../app-state";
import {Subject} from "rxjs";

@Component({
  selector: 'cre-nav',
  templateUrl: './nav.component.html',
  styleUrls: ['./nav.component.sass']
})
export class NavComponent implements OnInit, OnDestroy {
  private _destroy$ = new Subject<boolean>()

  @Input() links: NavLink [] = []

  _activeLink: string;

  constructor(
    private accountService: AccountService,
    private appState: AppState,
    private router: Router
  ) {
  }

  ngOnInit(): void {
    this._activeLink = this.router.url
    this.updateEnabledLinks(this.appState.authenticatedUser)

    this.appState.authenticatedUser$
      .pipe(takeUntil(this._destroy$))
      .subscribe({
        next: authentication => this.updateEnabledLinks(authentication.authenticatedUser)
      })
  }

  ngOnDestroy(): void {
    this._destroy$.next(true)
    this._destroy$.complete()
  }

  hasPermission(link: NavLink): boolean {
    return !link.permission || this.accountService.hasPermission(link.permission)
  }

  set activeLink(link: string) {
    this._activeLink = link
    this.router.navigate([link])
  }

  get activeLink() {
    return this._activeLink
  }

  private updateEnabledLinks(user: User) {
    this.links.forEach(l => {
      if (l.permission) {
        l.enabled = user && user.permissions.indexOf(l.permission) >= 0;
      }
    })
  }
}

export class NavLink {
  constructor(
    public route: string,
    public title: string,
    public permission?: Permission,
    public enabled?
  ) {
  }
}
