import {Injectable} from '@angular/core'
import {Config} from '../model/config.model'
import {Observable} from 'rxjs'
import {ApiService} from './api.service'
import {FormControl} from '@angular/forms'

const imageConfigsKeys = [
  Config.INSTANCE_LOGO_PATH,
  Config.INSTANCE_ICON_PATH
]

@Injectable({
  providedIn: 'root'
})
export class ConfigService {
  constructor(
    private api: ApiService
  ) {
  }

  get(key: string): Observable<Config> {
    return this.api.get<Config>(`/config/${key}`)
  }

  set(configs: Map<string, FormControl>): Observable<void> {
    const body = []
    for (let key in configs) {
      const control = configs[key]
      if (control.dirty && key.indexOf('path') < 0) {
        body.push({key, content: control.value})
      }
    }

    const subscriptions = []
    imageConfigsKeys.forEach(key => {
      if (configs[key].dirty) {
        subscriptions.push(this.setImage(key, configs[key].value))
      }
    })

    while (subscriptions.length > 0) {
      const subscription = subscriptions.pop().subscribe({
        next: () => subscription.unsubscribe()
      })
    }

    return this.api.put<void>('/config', body)
  }

  setImage(key: string, image: File): Observable<void> {
    const body = new FormData()
    body.append('key', key)
    body.append('image', image)

    return this.api.put<void>('/config/image', body)
  }

  restart(): Observable<void> {
    return this.api.post<void>('/config/restart')
  }
}
